{by David Ratcliff 'ratronic' 9/15/10}
Con
                                              
  _clkmode = xtal1 + pll16x                       'set propeller clock rate
  _xinfreq = 5_000_000                            '
  spd = 150                                       'servos moving speed
  dyn = 0                                         'prop i/o# for servos
  lwr = 200                                       'lower arm offset
'servo id#'s  1-base, 2&3-lower arm, 4&5 upper arm, 6-wrist, 7-gripper
  
Var

  long x, y, z, wrs                               'x - mouse/x, y - mouse/y, z - scroll wheel, wrs - wrist position
  byte grf, wrf                                   'grf - gripper flag,   wrf - wrist flag
  
Obj

  vga : "vga_text"                                'vga text display driver
   dy : "dynacomv1.2"                             'dynamixel servo driver
   ms : "mouse"                                   'mouse driver
   
Pub main | i

  'initialization
  wrs := 512                                      'midway wrist position 
  vga.start(16)                                   'start vga display driver
  ms.start(24, 25)                                'start mouse driver
  dy.start(dyn, 500_000)                          'start dynamixel servo bus driver on prop i/o @ .5m baud  
  ms.bound_scales(1, 1, 1)                        'set mouse bound scales             
  ms.bound_limits(0, -512, -512, 1023, 512, 512)  'set mouse xlimit 0 to 1023, ylimit -512 to 512, zlimit -512 to 512
  ms.bound_preset(512,0,0)                        'set mouse start preset x = 512, y = 0, z = 0
  repeat i from 1 to 7                            'set all servos moving speed
    dy.setsrvspd(i, spd)                          '
  dy.setsrvpos(6, wrs)                            'set wrist position
  dy.setsrvpos(7, 530)                            'set gripper open
  
  'main loop
  repeat
    x := ms.bound_x                               'assign current mouse x position to x
    y := ms.bound_y                               'assign current mouse y position to y
    z := ms.bound_z                               'assign current scroll wheel position to z
    dy.setsrvpos(1, x)                            'set arm base position to mouse x position
    syncmove(2, y + lwr)                          'set lower arm position to mouse y position + lower arm offset
    syncmove(4, y + (z * 5))                      'set upper arm position to mouse y position + (scroll wheel*5)
    if ms.buttons == 1                            'open/close gripper (left mouse button)
      gripper                                     '
    if ms.buttons == 2                            'toggle wrist rotation (right mouse button)
      wrist                                       '
    display                                       'display smart arm positions
           
Pub gripper                                       'open/close gripper on each call

  if grf                                          'open gripper
    dy.setsrvpos(7, 530)
    waitcnt(clkfreq + cnt)
    !grf
  else                                            'close gripper
    dy.setsrvpos(7, 750)
    waitcnt(clkfreq + cnt)
    dy.setsrvpos(7, dy.getsrvpos(7))
    !grf
    
Pub syncmove(srv, pos) | hpos1, lpos1, hpos2, lpos2, hspd, lspd, srv1, srv2 'move dual servo arm joints on smart arm
                                                                            'srv = 2 is lower arm                     
  hspd := spd / 256                                                         'srv = 4 is upper arm                     
  lspd := spd - (hspd * 256)                                                'pos = -512 to 512 possible arm positions 
  srv1 := 512 - pos                                                          
  srv2 := 512 + pos
  hpos1 := srv1 / 256           
  lpos1 := srv1 - (hpos1 * 256)     
  hpos2 := srv2 / 256
  lpos2 := srv2 - (hpos2 * 256)
  dy.setbabf(0, srv)           
  dy.setbabf(1, lpos1)      
  dy.setbabf(2, hpos1)       
  dy.setbabf(3, lspd)      
  dy.setbabf(4, hspd)       
  dy.setbabf(5, srv + 1)       
  dy.setbabf(6, lpos2)     
  dy.setbabf(7, hpos2)       
  dy.setbabf(8, lspd)      
  dy.setbabf(9, hspd)       
  dy.syncwrite(2, 30, 4)

Pub wrist                                         'move wrist opposite direction on each call
                                                                       
  if wrf                                          'turn wrist counter clockwise
    repeat
      dy.setsrvpos(6, wrs++)
      wrs :=  0 #> wrs <# 1023
      waitcnt(200_000 + cnt)
      if ms.buttons == 0
        !wrf
        return
  else                                            'turn wrist clockwise
    repeat
      dy.setsrvpos(6, wrs--)
      wrs :=  0 #> wrs <# 1023
      waitcnt(200_000 + cnt) 
      if ms.buttons == 0
        !wrf
        return

Pub display                                       'put current smart arm positions on display

  vga.out(1)
  vga.str(string("Smart arm positions",13,13))
  vga.str(string("Base      = "))
  vga.dec(x)
  vga.str(string("   ",13))  
  vga.str(string("Lower arm = "))
  vga.dec(y + lwr)
  vga.str(string("   ",13))
  vga.str(string("Upper arm = "))
  vga.dec(y + z)
  vga.str(string("   ",13))    
  vga.str(string("Wrist     = "))
  vga.dec(wrs)
  vga.str(string("   ",13))
  vga.str(string("Gripper   = "))
  if grf
    vga.str(string("Closed ",13))
  else
    vga.str(string("Opened ",13))
             